from time import sleep

from django.db import connections

from .Database import DatabaseStatus


class WatchDog():
    def __init__(self, databases,wait_time):
        self._databases=databases
        self._wait_time=wait_time

    def check_databases_statuses(self):
        while True:
            sleep(self._wait_time)
            for database in self._databases:
                try:
                    connections[database.name].connect()
                except database.operational_error:
                    database.status.put(DatabaseStatus.DOWN.value)
                    database.is_down.aquire()
                else:
                    database.status.put(DatabaseStatus.RUNNING.value)
                    database.is_down.release()



import enum


class Query:
    def __init__(self, wait, type, method=None, args=[], kwargs={}, model=None, query_id=None):
        self.method = method
        self.args = args
        self.kwargs = kwargs
        self.type = type
        self.wait = wait
        self.model = model
        self.query_id = query_id


class QueryType(enum.Enum):
    QUERYSET = "QUERYSET"
    NO_QUERYSET = "NO_QUERYSET"
    INFO_NUMBER_OF_CONNECTIONS = "INFO_NUMBER_OF_CONNECTIONS"
    INFO_RESPONSE_TIME = "INFO_RESPONSE_TIME"


class Wait(enum.Enum):
    WAIT = "WAIT"
    DONT_WAIT = "DONT_WAIT"



import django
import django.db

from .Database import DatabaseStatus


class Waiter:
    def __init__(self, executor=None):
        self._executor = executor

    def run_query(self, query, database):
        pass


class WaitWaiter(Waiter):
    def __init__(self, executor=None):
        super(WaitWaiter, self).__init__(executor)

    def run_query(self, query, database):
        while database.status.get() == DatabaseStatus.RUNNING.value:
            try:
                self._executor.run_query(query, database)
                break
            except django.db.utils.OperationalError:
                database.status.put(DatabaseStatus.DOWN.value)
                database.is_down.aquire()
                self.wait_for_connection(query, database)

    def wait_for_connection(self, query, database):
        while database.is_down.aquire():
            pass


class DontWaitWaiter(Waiter):
    def run_query(self, query, database):
        if database.status.get() == DatabaseStatus.RUNNING.value:
            try:
                self._executor.run_query(query, database)
            except django.db.utils.OperationalError:
                database.status.put(DatabaseStatus.DOWN.value)
                database.is_down.aquire()
                raise django.db.utils.OperationalError
